function [A,Lam,yclean]=sfactor(y,nl,nem)
%
%  programmed by K. Sekihara on March, 2015
%  All right reserved by K. Sekihara
%  {R[h̎gpɍۂĂREADME.pdfɂgpmFĂ
%
% Bayes factor analysis
%  
% M: data dimension (number of sensors)
% K: number of time points
% nl: number of factors
% nem: number of EM iterations
% y(M,K): data
% A(M:nl): mixing matrix
% lam(M,1): sensor noise precision (vector consisting of diagonal elements)
% Lam(M,M): sensor noise precision matrix (diagonalmatrix)
% ubar(nl,K): posterior mean of factor activity
% yclean(M,K): cleaned data
%
% initialization
%
  K=size(y,2);
%
% initialization
% A is initialized using the svd of the data 
 [U, S , ~]=svd(y,'econ');
 sd=diag(S);
 A=U(:,1:nl)*diag(sd(1:nl));
%
% The noise precision is initialized using the diagonal elements 
% of the data covariance
 Ryy=y*y';
 lam=1 ./diag(Ryy/K);
 Lam=diag(lam);        % diagonal matrix
%
% EM iteration
%
h = waitbar(0,'factor analysis EM');
 for iem=1:nem
%
%  e-step
%
   Gamma=A'*Lam*A+eye(nl,nl);           % Eq.(6.10)
   iGam=Gamma\eye(size(Gamma));
   ubar=iGam*A'*Lam*y;                  % Eq.(6.11)
%
% calculate sufficient statistics
%
  Ruu=ubar*ubar'+K*iGam;    % Eq.(6.15)
  Ruy=ubar*y';              % Eq.(6.16) and (6.17)
  Ryu=Ruy';
%
% m-step updates
%
  iRuu=Ruu\eye(size(Ruu));
  A=Ryu*iRuu;               % Eq.(6.18)
  
 ilam=diag(Ryy-A*Ruy)/K;   
 lam= 1 ./ilam;             % Eq.(6.23)
  Lam=diag(lam);
%
waitbar(iem/nem,h,'fa em iteration');
 end
 close(h)

yclean=A*ubar;


