function [A,Lam,yclean]=svbfa(y,nl,nem)
%  programmed by K. Sekihara on March, 2015
%  All right reserved by K. Sekihara
%  {R[h̎gpɍۂĂREADME.pdfɂgpmFĂ
%
% vb factor analysis
%  
% M: data dimension (number of sensors)
% K: number of time points
% nl: number of factors
% nem: number of EM iterations
% y(M,K): data
% A(M:nl): posterior mea of the mixing matrix
% lam(M,1): sensor noise precision (vector consisting of diagonal elements)
% Lam(M,M): sensor noise precision matrix (diagonal matrix)
% ubar(nl,K): posterior mean of factor activity
% yclean(M,K): cleaned data

 M=size(y,1);  K=size(y,2);
%
% initialization
% A is initialized using the svd of the data 
 [U, S , ~]=svd(y,'econ');
 sd=diag(S);
 A=U(:,1:nl)*diag(sd(1:nl));
%
% The noise precision is initialized using the diagonal elements 
% of the data covariance
 Ryy=y*y';
 lam=1 ./diag(Ryy/K);
 Lam=diag(lam);
%
 iPsi=eye(nl,nl);
 Alpha=diag(1./diag(A'*Lam*A/M+iPsi));
%
% EM iteration
%
% matrix preallocation
%
h = waitbar(0,'vbfa EM');
 for iem=1:nem
%
%  e-step
%
   Gamma=A'*Lam*A+eye(nl,nl)+M*iPsi;  % Eq.(8.21)
   iGam=Gamma\eye(size(Gamma));       %  iGam=inv(Gamma);
   ubar=iGam*A'*Lam*y;                % Eq.(8.22)
%
% calculate sufficient statistics
%
  Ruu=ubar*ubar'+K*iGam;             % Eq.(6.15)
  Ruy=ubar*y';                       % Eq.(6.16) and (6.17)
  Ryu=Ruy';
%
% m-step updates
%  mixing matrix
  Psi=Ruu+Alpha;  % Eq.(8.33)
  iPsi=Psi\eye(size(Psi));
  A=Ryu*iPsi;     % Eq.(8.31)
%
% noise precision: Eq.(8.47)
 ilam=diag(Ryy-A*Ruy)/K;
 lam= 1 ./ilam;
 Lam=diag(lam);
%
% hyperparameter:  Eq.(8.40)
 ialf=diag(A'*Lam*A/M+iPsi);
 Alpha=diag(1 ./ialf);
%
waitbar(iem/nem,h,'vbfa em iteration');
 end
close(h)

yclean=A*ubar;

